# -*- coding: utf-8 -*-
"""
Created on Sat Jun 14 14:17:41 2014

@author: aaron
"""
import sys
import subprocess

#subprocess.call([sys.executable, 'hiperwalk.py', '-test'])

#subprocess.call([sys.executable, 'hiperwalk.py', 'teste/coined1D.in'])
    #
#subprocess.call([sys.executable, 'hiperwalk.py', 'teste/coined2D.in'])
#
subprocess.call([sys.executable, 'hiperwalk.py', 'teste/staggered1D.in'])
#
#subprocess.call([sys.executable, 'hiperwalk.py', 'teste/staggered2D.in'])
#
#subprocess.call([sys.executable, 'hiperwalk.py', 'teste/custom.in'])

